@@ -644,6 +644,8 @@ parse (EVCard *evc, const char *str)
 		e_vcard_attribute_free (attr);
 
 	g_free (buf);
+
+	evc->priv->attributes = g_list_reverse (evc->priv->attributes);
 }
 
 /**
@@ -753,7 +755,7 @@ e_vcard_construct (EVCard *evc, const char *str)
  * Return value: A new, blank #EVCard.
  **/
 EVCard *
-e_vcard_new ()
+e_vcard_new (void)
 {
 	return e_vcard_new_from_string ("");
 }
@@ -804,10 +806,10 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 	g_string_append (str, "VERSION:3.0" CRLF);
 
 	for (l = evc->priv->attributes; l; l = l->next) {
-		GList *p;
+		GList *list;
 		EVCardAttribute *attr = l->data;
 		GString *attr_str;
-		int l;
+		glong len;
 
 		if (!g_ascii_strcasecmp (attr->name, "VERSION"))
 			continue;
@@ -826,8 +828,8 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 		g_string_append (attr_str, attr->name);
 
 		/* handle the parameters */
-		for (p = attr->params; p; p = p->next) {
-			EVCardAttributeParam *param = p->data;
+		for (list = attr->params; list; list = list->next) {
+			EVCardAttributeParam *param = list->data;
 			/* 5.8.2:
 			 * param        = param-name "=" param-value *("," param-value)
 			 */
@@ -837,14 +839,14 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 				g_string_append_c (attr_str, '=');
 				for (v = param->values; v; v = v->next) {
 					char *value = v->data;
-					char *p = value;
+					char *pval = value;
 					gboolean quotes = FALSE;
-					while (*p) {
-						if (!g_unichar_isalnum (g_utf8_get_char (p))) {
+					while (*pval) {
+						if (!g_unichar_isalnum (g_utf8_get_char (pval))) {
 							quotes = TRUE;
 							break;
 						}
-						p = g_utf8_next_char (p);
+						pval = g_utf8_next_char (pval);
 					}
 					if (quotes)
 						g_string_append_c (attr_str, '"');
@@ -883,21 +885,23 @@ e_vcard_to_string_vcard_30 (EVCard *evc)
 		 * When generating a content line, lines longer than 75
 		 * characters SHOULD be folded
 		 */
-		l = 0;
-		do {
-			
-			if ((g_utf8_strlen (attr_str->str, -1) -l) > 75) {
-				char *p;
-
-				l += 75;
-				p = g_utf8_offset_to_pointer (attr_str->str, l);
-			
-				g_string_insert_len (attr_str, (p - attr_str->str), CRLF " ", sizeof (CRLF " ") - 1);
-			}
-			else
-				break;
-		} while (l < g_utf8_strlen (attr_str->str, -1));
+		len = g_utf8_strlen (attr_str->str, -1);
+		if (len > 75) {
+			GString *fold_str = g_string_sized_new (attr_str->len + len/74*3);
+			gchar *pos1 = attr_str->str;
+			gchar *pos2 = pos1;
+			pos2 = g_utf8_offset_to_pointer (pos2, 75);
 
+			do {
+				g_string_append_len (fold_str, pos1, pos2 - pos1);
+				g_string_append (fold_str, CRLF " ");
+				pos1 = pos2;
+				pos2 = g_utf8_offset_to_pointer (pos2, 74);
+			} while (pos2 < attr_str->str + attr_str->len);
+			g_string_append (fold_str, pos1);
+			g_string_free (attr_str, TRUE);
+			attr_str = fold_str;
+		}
 		g_string_append (attr_str, CRLF);
 
 		g_string_append (str, attr_str->str);
@@ -996,7 +1000,7 @@ e_vcard_attribute_new (const char *attr_group, const char *attr_name)
 {
 	EVCardAttribute *attr;
 
-	attr = g_new0 (EVCardAttribute, 1);
+	attr = g_slice_new0 (EVCardAttribute);
 
 	attr->group = g_strdup (attr_group);
 	attr->name = g_strdup (attr_name);
@@ -1022,7 +1026,7 @@ e_vcard_attribute_free (EVCardAttribute *attr)
 
 	e_vcard_attribute_remove_params (attr);
 
-	g_free (attr);
+	g_slice_free (EVCardAttribute, attr);
 }
 
 /**
@@ -1041,8 +1045,7 @@ e_vcard_attribute_copy (EVCardAttribute *attr)
 
 	g_return_val_if_fail (attr != NULL, NULL);
 
-	a = e_vcard_attribute_new (e_vcard_attribute_get_group (attr),
-				   e_vcard_attribute_get_name (attr));
+	a = e_vcard_attribute_new (attr->group, attr->name);
 
 	for (p = attr->values; p; p = p->next)
 		e_vcard_attribute_add_value (a, p->data);
@@ -1123,7 +1126,7 @@ e_vcard_add_attribute (EVCard *evc, EVCardAttribute *attr)
 	g_return_if_fail (E_IS_VCARD (evc));
 	g_return_if_fail (attr != NULL);
 
-	evc->priv->attributes = g_list_append (evc->priv->attributes, attr);
+	evc->priv->attributes = g_list_prepend (evc->priv->attributes, attr);
 }
 
 /**
@@ -1230,7 +1233,7 @@ e_vcard_attribute_add_value_decoded (EVCardAttribute *attr, const char *value, i
 /**
  * e_vcard_attribute_add_values:
  * @attr: an #EVCardAttribute
- * ...: a %NULL-terminated list of strings
+ * @Varargs: a %NULL-terminated list of strings
  *
  * Adds a list of values to @attr.
  **/
@@ -1301,6 +1304,35 @@ e_vcard_attribute_remove_value (EVCardAttribute *attr, const char *s)
 	attr->values = g_list_delete_link (attr->values, l);
 }
 
+/**
+ * e_vcard_attribute_remove_param:
+ * @attr: an #EVCardAttribute
+ * @param_name: a parameter name
+ *
+ * Removes the parameter @param_name from the attribute @attr.
+ *
+ * Since 1.11.3.
+ */
+void
+e_vcard_attribute_remove_param (EVCardAttribute *attr, const char *param_name)
+{
+	GList *l;
+	EVCardAttributeParam *param;
+
+	g_return_if_fail (attr != NULL);
+	g_return_if_fail (param_name != NULL);
+	
+	for (l = attr->params; l; l = l->next) {
+		param = l->data;
+		if (g_ascii_strcasecmp (e_vcard_attribute_param_get_name (param),
+					param_name) == 0) {
+			attr->params = g_list_delete_link (attr->params, l);
+			e_vcard_attribute_param_free(param);
+			break;
+		}
+	}
+}
+
 /**
  * e_vcard_attribute_remove_params:
  * @attr: an #EVCardAttribute
@@ -1332,7 +1364,8 @@ e_vcard_attribute_remove_params (EVCardAttribute *attr)
 EVCardAttributeParam*
 e_vcard_attribute_param_new (const char *name)
 {
-	EVCardAttributeParam *param = g_new0 (EVCardAttributeParam, 1);
+	EVCardAttributeParam *param = g_slice_new (EVCardAttributeParam);
+	param->values = NULL;
 	param->name = g_strdup (name);
 
 	return param;
@@ -1353,7 +1386,7 @@ e_vcard_attribute_param_free (EVCardAttributeParam *param)
 
 	e_vcard_attribute_param_remove_values (param);
 
-	g_free (param);
+	g_slice_free (EVCardAttributeParam, param);
 }
 
 /**
@@ -1395,7 +1428,7 @@ e_vcard_attribute_add_param (EVCardAttribute *attr,
 	g_return_if_fail (attr != NULL);
 	g_return_if_fail (param != NULL);
 
-	attr->params = g_list_append (attr->params, param);
+	attr->params = g_list_prepend (attr->params, param);
 
 	/* we handle our special encoding stuff here */
 
@@ -1443,7 +1476,7 @@ e_vcard_attribute_param_add_value (EVCardAttributeParam *param,
 /**
  * e_vcard_attribute_param_add_values:
  * @param: an #EVCardAttributeParam
- * ...: a %NULL-terminated list of strings
+ * @Varargs: a %NULL-terminated list of strings
  *
  * Adds a list of values to @param.
  **/
@@ -1489,7 +1522,7 @@ e_vcard_attribute_add_param_with_value (EVCardAttribute *attr,
  * e_vcard_attribute_add_param_with_values:
  * @attr: an #EVCardAttribute
  * @param: an #EVCardAttributeParam
- * ...: a %NULL-terminated list of strings
+ * @Varargs: a %NULL-terminated list of strings
  *
  * Adds the list of values to @param, then adds @param
  * to @attr.
@@ -1587,19 +1620,29 @@ e_vcard_get_attributes (EVCard *evcard)
 	return evcard->priv->attributes;
 }
 
+/**
+ * e_vcard_get_attribute:
+ * @evc: an #EVCard
+ * @name: the name of the attribute to get
+ *
+ * Get the attribute @name from @evc.  The #EVCardAttribute is owned by
+ * @evcard and should not be freed. If the attribute does not exist, #NULL is
+ * returned.
+ *
+ * Return value: An #EVCardAttribute if found, or #NULL.
+ **/
 EVCardAttribute *
-e_vcard_get_attribute (EVCard     *vcard,
+e_vcard_get_attribute (EVCard     *evc,
 		       const char *name)
 {
         GList *attrs, *l;
 
-        g_return_val_if_fail (E_IS_VCARD (vcard), NULL);
+        g_return_val_if_fail (E_IS_VCARD (evc), NULL);
         g_return_val_if_fail (name != NULL, NULL);
 
-        attrs = e_vcard_get_attributes (vcard);
+        attrs = e_vcard_get_attributes (evc);
         for (l = attrs; l; l = l->next) {
                 EVCardAttribute *attr;
-                const char *n;
 		
                 attr = (EVCardAttribute *) l->data;
                 if (strcmp (attr->name, name) == 0)
@@ -1677,15 +1720,17 @@ e_vcard_attribute_get_values_decoded (EVCardAttribute *attr)
 		switch (attr->encoding) {
 		case EVC_ENCODING_RAW:
 			for (l = attr->values; l; l = l->next)
-				attr->decoded_values = g_list_append (attr->decoded_values, g_string_new ((char*)l->data));
+				attr->decoded_values = g_list_prepend (attr->decoded_values, g_string_new ((char*)l->data));
+			attr->decoded_values = g_list_reverse (attr->decoded_values);
 			break;
 		case EVC_ENCODING_BASE64:
 			for (l = attr->values; l; l = l->next) {
 				char *decoded = g_strdup ((char*)l->data);
 				int len = _evc_base64_decode_simple (decoded, strlen (decoded));
-				attr->decoded_values = g_list_append (attr->decoded_values, g_string_new_len (decoded, len));
+				attr->decoded_values = g_list_prepend (attr->decoded_values, g_string_new_len (decoded, len));
 				g_free (decoded);
 			}
+			attr->decoded_values = g_list_reverse (attr->decoded_values);
 			break;
 		case EVC_ENCODING_QP:
 			g_warning ("need to implement quoted printable decoding");
@@ -1838,8 +1883,8 @@ e_vcard_attribute_get_param (EVCardAttribute *attr, const char *name)
 {
 	GList *params, *p;
 	
-	g_return_val_if_fail (attr != NULL, FALSE);
-	g_return_val_if_fail (name != NULL, FALSE);
+	g_return_val_if_fail (attr != NULL, NULL);
+	g_return_val_if_fail (name != NULL, NULL);
 	
 	params = e_vcard_attribute_get_params (attr);
 
@@ -2093,7 +2138,7 @@ _evc_base64_encode_simple (const char *data, size_t len)
 {
 	unsigned char *out;
 	int state = 0, outlen;
-	unsigned int save = 0;
+	int save = 0;
 
 	g_return_val_if_fail (data != NULL, NULL);
 
