@@ -43,18 +43,7 @@ e_uid_new (void)
 	static char *hostname;
 
 	if (!hostname) {
-#if GLIB_CHECK_VERSION (2, 8, 0)
 		hostname = (char *) g_get_host_name ();
-#else
-		static char buffer [512];
-
-		if ((gethostname (buffer, sizeof (buffer) - 1) == 0) &&
-		    (buffer [0] != 0))
-			hostname = buffer;
-		else
-			hostname = "localhost";
-
-#endif
 	}
 
 	return g_strdup_printf ("%lu.%lu.%d@%s",
